"use strict";

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _asyncToGenerator2 = require("babel-runtime/helpers/asyncToGenerator");

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _bluebird = require("bluebird");

var _bluebird2 = _interopRequireDefault(_bluebird);

var _capitalize = require("capitalize");

var _capitalize2 = _interopRequireDefault(_capitalize);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/*  the mixin class  */
/*
**  GraphQL-Tools-Sequelize -- Integration of GraphQL-Tools and Sequelize ORM
**  Copyright (c) 2016-2018 Ralf S. Engelschall <rse@engelschall.com>
**
**  Permission is hereby granted, free of charge, to any person obtaining
**  a copy of this software and associated documentation files (the
**  "Software"), to deal in the Software without restriction, including
**  without limitation the rights to use, copy, modify, merge, publish,
**  distribute, sublicense, and/or sell copies of the Software, and to
**  permit persons to whom the Software is furnished to do so, subject to
**  the following conditions:
**
**  The above copyright notice and this permission notice shall be included
**  in all copies or substantial portions of the Software.
**
**  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
**  EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
**  MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
**  IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
**  CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
**  TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
**  SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/

/*  external dependencies  */
class gtsEntityQuery {
    /*  API: query/read one or many entities (directly or via relation)  */
    entityQuerySchema(source, relation, target) {
        let isMany = false;
        let m;
        if ((m = target.match(/^(.+)\*$/)) !== null) {
            target = m[1];
            isMany = true;
        }
        if (isMany) {
            /*  MANY  */
            if (relation === "")
                /*  directly  */
                return "" + `# Query one or many [${target}]() entities,\n` + "# by either an (optionally available) full-text-search (`query`)\n" + "# or an (always available) attribute-based condition (`where`),\n" + "# optionally filter them by a condition on some relationships (`include`),\n" + "# optionally sort them (`order`),\n" + "# optionally start the result set at the n-th entity (zero-based `offset`), and\n" + "# optionally reduce the result set to a maximum number of entities (`limit`).\n" + `${target}s(fts: String, where: JSON, include: JSON, order: JSON, offset: Int = 0, limit: Int = 100): [${target}]!\n`;else
                /*  via relation  */
                return "" + `# Query one or many [${target}]() entities\n` + `# by following the **${relation}** relation of [${source}]() entity,\n` + "# optionally filter them by a condition (`where`),\n" + "# optionally filter them by a condition on some relationships (`include`),\n" + "# optionally sort them (`order`),\n" + "# optionally start the result set at the n-th entity (zero-based `offset`), and\n" + "# optionally reduce the result set to a maximum number of entities (`limit`).\n" + `${relation}(where: JSON, include: JSON, order: JSON, offset: Int = 0, limit: Int = 100): [${target}]!\n`;
        } else {
            /*  ONE  */
            if (relation === "")
                /*  directly  */
                return "" + `# Query one [${target}]() entity by its unique identifier (\`id\`) or condition (\`where\`) or` + `# open an anonymous context for the [${target}]() entity.\n` + `# The [${target}]() entity can be optionally filtered by a condition on some relationships (\`include\`).\n` + `${target}(id: ${this._idtype}, where: JSON, include: JSON): ${target}\n`;else
                /*  via relation  */
                return "" + `# Query one [${target}]() entity by following the **${relation}** relation of [${source}]() entity.\n` + `# The [${target}]() entity can be optionally filtered by a condition (\`where\`).\n` + `# The [${target}]() entity can be optionally filtered by a condition on some relationships (\`include\`).\n` + `${relation}(where: JSON, include: JSON): ${target}\n`;
        }
    }
    entityQueryResolver(source, relation, target) {
        var _this = this;

        let isMany = false;
        let m;
        if ((m = target.match(/^(.+)\*$/)) !== null) {
            target = m[1];
            isMany = true;
        }
        return (() => {
            var _ref = (0, _asyncToGenerator3.default)(function* (parent, args, ctx, info) {
                if (isMany) {
                    /*  MANY  */

                    /*  determine filter options  */
                    let opts = _this._findManyOptions(target, args, info);
                    if (ctx.tx !== undefined) opts.transaction = ctx.tx;

                    /*  find entities  */
                    let objs;
                    if (relation === "") {
                        /*  directly  */
                        if (args.fts !== undefined)
                            /*  directly, via FTS index  */
                            objs = yield _this._ftsSearch(target, args.fts, args.order, args.offset, args.limit, ctx);else
                            /*  directly, via database  */
                            objs = yield _this._models[target].findAll(opts);
                    } else {
                        /*  via relation  */
                        let getter = `get${(0, _capitalize2.default)(relation)}`;
                        objs = yield parent[getter](opts);
                    }

                    /*  check authorization  */
                    objs = yield _bluebird2.default.filter(objs, function (obj) {
                        return _this._authorized("after", "read", target, obj, ctx);
                    });

                    /*  map field values  */
                    yield _bluebird2.default.each(objs, function (obj) {
                        _this._mapFieldValues(target, obj, ctx, info);
                    });

                    /*  trace access  */
                    yield _bluebird2.default.each(objs, function (obj) {
                        return _this._trace(target, obj.id, obj, "read", relation === "" ? "direct" : "relation", "many", ctx);
                    });

                    return objs;
                } else {
                    /*  ONE  */

                    /*  determine filter options  */
                    let opts = _this._findOneOptions(target, args, info);
                    if (ctx.tx !== undefined) opts.transaction = ctx.tx;

                    /*  find entity  */
                    let obj;
                    if (relation === "") {
                        /*  directly  */
                        if (args.id !== undefined)
                            /*  regular case: non-anonymous context, find by identifier  */
                            obj = yield _this._models[target].findById(args.id, opts);else if (args.where !== undefined)
                            /*  regular case: non-anonymous context, find by condition  */
                            obj = yield _this._models[target].findOne(opts);else
                            /*  special case: anonymous context  */
                            return new _this._anonCtx(target);
                    } else {
                        /*  via relation  */
                        let getter = `get${(0, _capitalize2.default)(relation)}`;
                        obj = yield parent[getter](opts);
                    }
                    if (obj === null) return null;

                    /*  check authorization  */
                    if (!(yield _this._authorized("after", "read", target, obj, ctx))) return null;

                    /*  map field values  */
                    _this._mapFieldValues(target, obj, ctx, info);

                    /*  trace access  */
                    yield _this._trace(target, obj.id, obj, "read", relation === "" ? "direct" : "relation", "one", ctx);

                    return obj;
                }
            });

            return function (_x, _x2, _x3, _x4) {
                return _ref.apply(this, arguments);
            };
        })();
    }
}
exports.default = gtsEntityQuery;
